<?php
/**
 * Namespace LancamentoContas
 * @category PHP
 * @package Financeiro\Logic\ContasPagar\LancamentoContas
 * @author  fabiooliveira <fabio.oliveira@teknisa.com>
 * @license http://erp.pebbian.com teknisa
 * @link    http://erp.pebbian.com
 */
namespace Financeiro\Logic\ContasPagar\LancamentoContas;
use \Fiscal\Entities\Contapagar;
use \Fiscal\Entities\Notafiscal;
use \Geral\Util\DateUtil;
use \Fiscal\Entities\Boletpagar;
use \Financeiro\Entities\Ajbolpag;
use \Geral\Util\NovoCodigo;
use \Geral\Util\Ambiente;
use \Financeiro\Entities\GeralPag;
use \Financeiro\Entities\Ratctpagar;
use \Setforms2\Runtime\Helper\Procedure;
use \Setforms2\Runtime\Helper\Procedure\Param;
use \Financeiro\Entities\Ajctpagar;
use \Contabilidade\Entities\Contopectbl;
use \Financeiro\Logic\AtualizaFluxodeCaixa\FluxodeCaixa;
use Setforms2\Shared\Registry;

/**
 * Class LancamentoContas
 * @category PHP
 * @package Financeiro\Logic\ContasPagar\LancamentoContas
 * @author  fabiooliveira <fabio.oliveira@teknisa.com>
 * @license http://erp.pebbian.com teknisa
 * @link    http://erp.pebbian.com
 */
class LancamentoContas
{
    const IDATIVO = 'S';

    /**
     * Método que cria novo boleto com seus respectivos valores,
     * @param Array $campos Campos
     * @return int|string
     * @throws \Exception
     */
    public static function inserirBoleto($campos)
    {
        try{
            $nrorg = Ambiente::getNrorg();
            $nrlanbolpag = NovoCodigo::geraCodigo(Ambiente::getNrorg(), 'BOLETPAGAR'.Ambiente::getCdFilial(), 1, 10);
            $boletpagar = new Boletpagar();
            $boletpagar->setCdforneced($campos["CDFORNECED"]);
            $boletpagar->setCdempresa($campos["CDEMPRESA"]);
            $boletpagar->setVrbolpag($campos["VRBOLPAG"]);
            $boletpagar->setVrfinalbpag($campos["VRFINALBPAG"]);
            $data = self::formataData($campos["DTVENBOLPAG"]);
            $boletpagar->setDtvenbolpag($data);
            $boletpagar->setCdfilial(Ambiente::getCdFilial());
            $boletpagar->setNrlanbolpag($nrlanbolpag);
            $boletpagar->setIdhabibpag('N');
            $boletpagar->setIdbpagreal('N');
            $boletpagar->setNrparcbolpag('00');
            $boletpagar->setDtinclusao(DateUtil::getDataAtual());
            $boletpagar->setNrorg($nrorg);
            $boletpagar->setDtultatu(DateUtil::getDataAtual());
            $boletpagar->setCdoperinclusao(Ambiente::getCdOperador());
            $boletpagar->setCdoperultatu(Ambiente::getCdOperador());
            $boletpagar->setIdativo(self::IDATIVO);
            $boletpagar->persist();
            Registry::getEntityManager()->flush();
                return $nrlanbolpag;
        } catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * @param $campos
     * @param $vrratccctpag
     * @param $cdcentcust
     * @return Null
     * @throws \Exception
     */
    public static function inserirRateio($campos, $vrratccctpag, $cdcentcust)
    {
        try{
            $rateiopagar = Ratctpagar::findOneByNrorgAndNrlanctonfAndCdcentcustAndNrparcelapagAndNrseqvenpag($campos['NRORG'], $campos['NRLANCTONF'], $cdcentcust, $campos['NRPARCELAPAG'], $campos['NRSEQVENPAG']);
            if (!$rateiopagar) {
                $rateiopagar = new Ratctpagar();
            }
            $rateiopagar->setCdcentcust($cdcentcust);
            $rateiopagar->setVrratccctpag($vrratccctpag);
            $rateiopagar->setCdfilial($campos['CDFILIAL']);
            $rateiopagar->setNrlanctonf($campos['NRLANCTONF']);
            $rateiopagar->setNrseqvenpag($campos['NRSEQVENPAG']);
            $rateiopagar->setNrparcelapag($campos['NRPARCELAPAG']);
            $rateiopagar->setCdfiratctpag($campos['CDFIRATCTPAG']);
            $rateiopagar->setNrorg(Ambiente::getNrorg());
            $now = self::formataData(now());
            $rateiopagar->setDtinclusao($now);
            $rateiopagar->persist();
            Registry::getEntityManager()->flush();
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * Metodo
     * @param array $campos Campos
     * @return Null
     * @throws \Exception
     */
    public static function inserirConta($campos)
    {
        try{
            $nrorg = Ambiente::getNrorg();
            $cdfilial = Ambiente::getCdFilial();
            $contapagar = Contapagar::getObjectByWhere('NRLANCTONF = ? AND NRORG = ? AND NRSEQVENPAG = ? AND CDFILIAL = ? AND NRPARCELAPAG = ?', array($campos["NRLANCTONF"], $nrorg, $campos["NRSEQVENPAG"], $cdfilial,  $campos["P_NRPARCELAPAG"]));

            if (!$contapagar && (($campos["NRPARCELAPAG"] > '01') || ($campos["P_NRPARCELAPAG"] == '00'))) {
                $contapagar = new Contapagar();
                $contapagar->setQtdiasvenpag(0);
            }

            if ($campos["NRPARCELAPAG"] == '01') {
                $rateio = Ratctpagar::getObjectByWhere(
                    'NRLANCTONF = ? AND NRSEQVENPAG = ? AND CDFILIAL = ? AND NRPARCELAPAG = ? AND NRORG = ?',
                    array($campos["NRLANCTONF"], $campos["NRSEQVENPAG"], $cdfilial,  '00', $campos['NRORG']));
                if ($rateio) {
                    $rateio->delete();
                    Registry::getEntityManager()->flush();
                }
            }

            $contapagar->setCdfilial($cdfilial);
            $contapagar->setNrlanctonf($campos["NRLANCTONF"]);
            $contapagar->setNrseqvenpag($campos["NRSEQVENPAG"]);
            $contapagar->setVratuapag($campos["VRATUAPAG"]);
            $data = self::formataData($campos["DTATUAVENPAG"]);
            $contapagar->setDtatuavenpag($data);
            $data1 = self::formataData($campos["DTORIGVENPAG"]);
            $contapagar->setDtorigvenpag($data1);
            $contapagar->setDscontapag($campos["DSCONTAPAG"]);
            $contapagar->setCdtpctpagar($campos["CDTPCTPAGAR"]);
            $contapagar->setQtdiasvenpag($campos["QTDIASVENPAG"]);
            $contapagar->setNrparcelapag($campos["NRPARCELAPAG"]);
            $contapagar->setIdstatpag($campos["IDSTATPAG"]);
            $contapagar->setCdfilibpag($campos["CDFILIBPAG"]);
            $contapagar->setNrlanbolpag($campos["NRLANBOLPAG"]);
            $contapagar->setVrorigpag($campos["VRORIGPAG"]);
            $contapagar->setCdclassfina($campos["CDCLASSFINA"]);
            $contapagar->setCdempresa($campos["CDEMPRESA"]);
            $contapagar->setCdfimovflpag($campos["CDFIMOVFLPAG"]);
            $data2 = self::formataData($campos["DTMOVFLUXO"]);
            $contapagar->setDtmovfluxo($data2);
            $contapagar->setIdmovfluxo($campos["IDMOVFLUXO"]);
            $contapagar->setNrorg($nrorg);
            $contapagar->setDtinclusao(DateUtil::getDataAtual());
            if($campos['CDMOTIPARC'])
                $contapagar->setCdmotiparc($campos['CDMOTIPARC']);
            $contapagar->persist();

            Registry::getEntityManager()->flush();
        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * Metodo
     * @param Int      $cdfilial     Cdfilial
     * @param Int      $nrlanctonf   Nrlanctonf
     * @param Int      $nrparcelapag Nrparcelapag
     * @param Int      $nrseqvenpag  Nrseqvenpag
     * @param String   $idtipo       Idtipo
     * @param Int      $cdajuctpag   Cdajuctpag
     * @param DateTime $dtajustepag  Dtajustepag
     * @return mixed
     * @throws \Exception
     */
    public static function contabilizaLancamento($cdfilial, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $idtipo, $cdajuctpag, $dtajustepag)
    {
        try{
            $nrlancctb = '';
            $procedure = new Procedure('CTB_PAG_LANC');
            $procedure->addParam(new Param('P_CDFILIAL', Param::PARAM_INPUT, $cdfilial, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTONF', Param::PARAM_INPUT, $nrlanctonf, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRPARCELAPAG', Param::PARAM_INPUT, $nrparcelapag, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRSEQVENPAG', Param::PARAM_INPUT, $nrseqvenpag, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTAJUSTEPAG', Param::PARAM_INPUT, $dtajustepag, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDAJUCTPAG', Param::PARAM_INPUT, $cdajuctpag, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDTIPO', Param::PARAM_INPUT, $idtipo, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCCTB', Param::PARAM_OUTPUT, $nrlancctb, Param::PARAM_TYPE_STR, 12));
            $nrlancctb = $procedure->call();
            return $nrlancctb['P_NRLANCCTB'];
        }
        catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * Metodo
     * @param Array $campos Campos
     * @return Null
     * @throws \Exception
     */
    public static function updateNotaFiscal($campos)
    {
        try {
            $notafiscal = Notafiscal::findOneByCdfilialAndNrlanctonfAndNrorg($campos['CDFILIAL'], $campos['NRLANCTONF'], $campos['NRORG']);
            $notafiscal->setVrnotafisc($campos['VRNOTAFISC']);
            $notafiscal->persist();
            Registry::getEntityManager()->flush();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Metodo
     * @param String $filial      Filial
     * @param Int    $nrlanbolpag Nrlanbolpag
     * @return mixed
     * @throws \Exception
     */
    public static function buscaAjustes($filial, $nrlanbolpag, $nrorg)
    {
        try {
            $objeto = Ajbolpag::findOneByNrlanbolpagAndCdfilialAndNrorg($nrlanbolpag, $filial, $nrorg);
            return $objeto;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Metodo
     * @param Int      $cdempresa   Cdempresa
     * @param Int      $cdfilial    Cdfilial
     * @param Int      $cdclassFina CdclassFina
     * @param DateTime $dtMovFluxo  Dtmovfluxo
     * @param Double   $valor       Valor
     * @param String   $idtipomov   Idtipomov
     * @param String   $idOperacao  Idoperacao
     * @return Null
     * @throws \Exception
     */
    public static function atualizaFluxodeCaixa($cdempresa, $cdfilial, $cdclassFina, $dtMovFluxo, $valor, $idtipomov, $idOperacao)
    {
        try{
            $connection = Registry::getEntityManager();
            $connection->beginTransaction();

            FluxodeCaixa::atualizaFluxo($cdempresa, $cdfilial, $cdclassFina, $dtMovFluxo, $valor, $idtipomov, $idOperacao);

            $connection->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    public static function updateBoletoeConta($nrorg, $cdfilial, $nrlanbolpag, $nrlanctonf, $nrparcelapag, $nrseqvenpag, $valor){
        try {
            $connection = Registry::getEntityManager();
            $connection->beginTransaction();
            /** @var $boleto Boletpagar */

            $boleto = Boletpagar::findOneByCdfilialAndNrlanbolpag($cdfilial, $nrlanbolpag, $nrorg);
            $boleto->setVrbolpag($valor);
            $boleto->setVrfinalbpag($valor);
            $boleto->persist();
            /** @var $conta Contapagar */
            $conta = Contapagar::findOneByNrorgAndCdfilialAndNrlanctonfAndNrparcelapagAndNrseqvenpag($nrorg, tk_get_field_value('CDFILIAL'), $nrlanctonf, $nrparcelapag, $nrseqvenpag);
            $conta->setVratuapag($valor);
            $conta->persist();

            $connection->flush();
            $connection->commit();
            tk_reload_data_with_filter();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function validaAjuste($cdfilial, $nrlanctonf, $nrorg){
        try{
            /** @var $ajuste Ajctpagar */
            $ajuste = Ajctpagar::findOneByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, $nrorg);
            if($ajuste)
                throw new \Exception(tk_get_message_by_name('msg_pbi00015_contapossuiajuste'));
        }catch(\Exception $e){
            throw $e;
        }
    }

    public static function deleteAjusteConta($cdfilial, $nrlanctonf, $nrorg){
        try{
            /** @var $ajuste Ajctpagar */
            $ajustes = Ajctpagar::findByCdfilialAndNrlanctonfAndNrorg($cdfilial, $nrlanctonf, $nrorg);
            foreach($ajustes as $ajuste) {
               $ajuste->delete();
            }

        }catch(\Exception $e){
            throw $e;
        }
    }

    /**
     * @param $nrlanbolpag
     * @param $cdfilial
     * @param $nrorg
     * @throws \Exception
     * Busca possíveis ajustes de um boleto e os excluí caso existam
     */
    public static function deleteAjustesBoleto($nrlanbolpag, $cdfilial, $nrorg)
    {
        try {
            /** @var $boleto Ajbolpag */
            $ajustes = Ajbolpag::findByNrlanbolpagAndCdfilialAndNrorg($nrlanbolpag, $cdfilial, $nrorg);
            if ($ajustes) {
                /** @var  $ajuste Ajbolpag*/
                foreach($ajustes as $ajuste) {
                   $ajuste->delete();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function verificaAjusteBoleto($nrlanbolpag, $cdfilial, $nrorg){
        try{
            /** @var $boleto Ajbolpag */
            $ajuste = Ajbolpag::findOneByNrlanbolpagAndCdfilialAndNrorg($nrlanbolpag, $cdfilial, $nrorg);
            if($ajuste){
                throw new \Exception(tk_get_message_by_name('msg_pbi00015_verificaajusteboleto'));
            }
        } catch(\Exception $e){
            throw $e;
        }
    }

    public static function validaBoletoCnab($nrlanbolpag, $cdfilial, $nrorg){
        try{
            /** @var $boleto Boletpagar */
            $boleto = Boletpagar::findOneByCdfilialAndNrlanbolpagAndNrorg($cdfilial, $nrlanbolpag, $nrorg);
            if($boleto)
                if($boleto->getNrcnabpagrem())
                    throw new \Exception(tk_get_message_by_name('msg_pbi00015_validaboletocnab'));
        } catch(\Exception $e){
            throw $e;
        }
    }

    public static function validaLancamento($cdempresa, $data, $nrorg){
        try{
            /** @var $ajuste \Contabilidade\Repositories\Contopectbl */
            $ajuste = Contopectbl::getRepository();
            $data = $ajuste->vdPeriodoLancamento($cdempresa, $data, $nrorg);
            if($data == null)
                throw new \Exception(tk_get_message_by_name('msg_pbi00015_validaboletocnab'));
        }catch(\Exception $e){
            throw $e;
        }
    }

    public static function buscaParamDatas(){
        try{
            $nrorg = Ambiente::getNrorg();
            /** @var $object GeralPag */
            $object = GeralPag::getObjectByWhere('NRORG = ?', array($nrorg));

            $array = array(
                'ANT' => $object->getQtdiaantbaip(),
                'APO' => $object->getQtdiaapobaip()
            );
            return $array;
        }catch(\Exception $e){
            throw $e;
        }
    }

    public static function formataData($data){
        if(strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }

    public static function updateRateio($rat, $vrratccctpag){
        try {
            $connection = Registry::getEntityManager();
            $connection->beginTransaction();

            $rat->setVrratccctpag($vrratccctpag);

            $connection->flush();
            $connection->commit();

        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function validaBoletoAssociado($nrorg, $nrlanbolpag, $cdfilial)
    {
        try {
            $boleto = Contapagar::findByNrorgAndNrlanbolpagAndCdfilial($nrorg, $nrlanbolpag, $cdfilial);
            if (count($boleto) > 1)
                throw new \Exception('Conta relacionada a um boleto real não pode ser alterada.');
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public static function updateProrrogacao(
        $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag, $cdfilibpag, $nrlanbolpag, $vencimento, $cdmotipror
    )
    {
        $connection = Registry::getEntityManager();
        $connection->beginTransaction();
        try {
            /** @var $boleto Boletpagar */
            $boleto = Boletpagar::findOneByNrorgAndCdfilialAndNrlanbolpag($nrorg, $cdfilibpag, $nrlanbolpag);
            $vencimento = self::formataData($vencimento);
            $boleto->setDtvenbolpag($vencimento);

            /** @var $contapagar Contapagar */
            $contapagar = Contapagar::findOneByNrorgAndCdfilialAndNrlanctonfAndNrseqvenpagAndNrparcelapag(
                $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag, $nrparcelapag
            );
            $contapagar->setDtatuavenpag($vencimento);
            $contapagar->setCdmotipror($cdmotipror);

            $connection->flush();
            $connection->commit();
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public static function geraRateio($cdcentcust, $vrratccctpag, $nrorg, $cdfilial, $nrlanctonf, $nrseqvenpag,
                                      $nrparcelapag, $cdfiratctpag){
        try{
            $connection = Registry::getEntityManager();
            $connection->beginTransaction();
            $rateiopagar = Ratctpagar::findOneByNrorgAndNrlanctonfAndCdcentcustAndNrparcelapagAndNrseqvenpag($nrorg, $nrlanctonf, $cdcentcust, $nrparcelapag, $nrseqvenpag);
            if(!$rateiopagar){
                $rateiopagar = new Ratctpagar();
            }
            $rateiopagar->setCdcentcust($cdcentcust);
            $rateiopagar->setVrratccctpag($vrratccctpag);
            $rateiopagar->setNrorg($nrorg);
            $rateiopagar->setCdfilial($cdfilial);
            $rateiopagar->setNrlanctonf($nrlanctonf);
            $rateiopagar->setNrseqvenpag($nrseqvenpag);
            $rateiopagar->setNrparcelapag($nrparcelapag);
            $rateiopagar->setCdfiratctpag($cdfiratctpag);
            $now = self::formataData(now());
            $rateiopagar->setDtinclusao($now);
            $rateiopagar->persist();

            $connection->flush();
            $connection->commit();
        }catch(\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }
	}
